/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.ant;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.apache.tools.ant.BuildException;

/**
 * This class is used to gather operations on all service assemblies
 * 
 * @author ofabre - eBMWebsourcing
 */
public abstract class AbstractSAAdminAntTask extends
    AbstractJBIAntTask {

    /**
     * Action : shutdown all components
     */
    protected static final String SHUTDOWN_ALL_SA = "shutdownAllServiceAssemblies";

    /**
     * Action : start all components
     */
    protected static final String START_ALL_SA = "startAllServiceAssemblies";

    /**
     * Action : stop all components
     */
    protected static final String STOP_ALL_SA = "stopAllServiceAssemblies";

    /**
     * Action : Uninstall all components
     */
    protected static final String UNDEPLOY_ALL_SA = "undeployAllServiceAssemblies";

    /**
     * Boolean specifying if installer unloading must be executed in any state
     */
    protected boolean force;

    /**
     * The action to perform
     */
    private String action;

    /**
     * Creates a new instance of {@link AbstractSAAdminAntTask}
     * 
     * @param action
     */
    public AbstractSAAdminAntTask(String action) {
        this.action = action;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.ant.managers.AbstractJBIAntTask#doTask()
     */
    public void doTask() throws Exception {

        if (UNDEPLOY_ALL_SA.equalsIgnoreCase(action)) {
            performDeploy();
        } else {
            performAction();
        }
    }

    /**
     * Perform an action on all service assemblies
     * 
     * @param action
     *            action to perform
     */
    protected void performAction() throws Exception {
        MBeanServerConnection connection = connector.getMBeanServerConnection();

        Object[] parameters = new Object[0];
        String[] signature = new String[0];
        ObjectName objectName = JBIJMXConnectorUtil
            .getDeploymentServiceMBeanName(connection);

        if (objectName == null) {
            throw new BuildException(
                "The Deployment Service is not accessible");
        }

        try {
            log("Invoking '" + action + "' on Deployment Service");
        } catch (NullPointerException e) {
            // Exception thrown when execute outisde of the ant context
        }

        connection.invoke(objectName, action, parameters, signature);
    }

    /**
     * Undeploy all service assemblies
     * 
     * @throws Exception
     */
    protected void performDeploy() throws Exception {
        MBeanServerConnection connection = connector.getMBeanServerConnection();

        Object[] parameters1 = new Object[1];
        parameters1[0] = force;
        String[] signature1 = new String[] {"boolean"};

        try {
            log("Invoking '" + action + "' on Deployment Service");
        } catch (NullPointerException e) {
            // Exception thrown when execute outisde of the ant context
        }
        connection.invoke(JBIJMXConnectorUtil
            .getDeploymentServiceMBeanName(connection), action, parameters1,
            signature1);

    }

    public void setForce(boolean force) {
        this.force = force;
    }

}
